package com.virjar.echo.nat.protocol;

import com.virjar.echo.nat.log.EchoLogger;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;

import java.util.List;

public class EchoPacketDecoder extends ByteToMessageDecoder {

    private static final byte HEADER_SIZE = 12;

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {
        EchoLogger.getLogger().info("decode data for: " + ctx.channel());
        while (true) {
            if (in.readableBytes() < HEADER_SIZE) {
                return;
            }

            int originIndex = in.readerIndex();
            long magic = in.readLong();
            if (magic != PacketCommon.magic) {
                ctx.close();
                throw new IllegalStateException("package not start with legal magic: " + magic + " expected:" + PacketCommon.magic);
            }

            int frameLength = in.readInt();
            if (in.readableBytes() < frameLength) {
                EchoLogger.getLogger().info("not enough data received: readableBytes->" + in.readableBytes()
                        + " frameLength->" + frameLength
                );
                in.readerIndex(originIndex);
                return;
            }

            try {
                EchoPacket echoPacket = new EchoPacket();
                byte type = in.readByte();
                long sn = in.readLong();

                echoPacket.setSerialNumber(sn);
                echoPacket.setType(type);

                byte extraByteLength = in.readByte();
                byte[] extraBytes = new byte[extraByteLength];
                in.readBytes(extraBytes);
                echoPacket.setExtra(new String(extraBytes));

                int dataLength = frameLength - PacketCommon.TYPE_SIZE - PacketCommon.SERIAL_NUMBER_SIZE - PacketCommon.EXTRA_LENGTH_SIZE - extraByteLength;
                if (dataLength < 0) {
                    EchoLogger.getLogger().error("message protocol error,negative data length:" + dataLength + " for channel: " + ctx.channel()
                            + " frameLength: " + frameLength + " type:" + type + " serial_number:" + sn + " uriLength:" + extraByteLength + " extra:" + echoPacket.getExtra()
                    );
                    ctx.channel().close();
                    return;
                }
                if (dataLength > 0) {
                    byte[] data = new byte[dataLength];
                    in.readBytes(data);
                    echoPacket.setData(data);
                }

                out.add(echoPacket);

            } catch (Exception e) {
                EchoLogger.getLogger().error("message decode failed for channel: " + ctx.channel(), e);
                ctx.channel().close();
            }

        }
    }

}
